/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.engine.regex;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import benten.cat.tm.core.BentenTmEngine;
import benten.cat.tm.core.BentenTmSearchResult;
import benten.core.BentenConstants;
import blanco.tmx.BlancoTmxParser;
import blanco.tmx.valueobject.BlancoTmx;
import blanco.tmx.valueobject.BlancoTmxTu;
import blanco.tmx.valueobject.BlancoTmxTuv;

/**
 * このクラスは、TM エンジンの切り替えテストのためのテストクラスです。
 * 実際の Benten 基本セットには含まれません。
 */
public class RegexTmEngine implements BentenTmEngine {
	private List<TmTuvEntry> fListTuvEntry = new ArrayList<TmTuvEntry>();

	private static Pattern extensionPattern = Pattern.compile("\\.[^.]*$"); //$NON-NLS-1$

	private static final int MAX_RESULT_SIZE = 3000;

	public RegexTmEngine() {
	}

	public void loadTmx(File dirTmx) throws IOException {
		final File[] fileTmxs = dirTmx.listFiles();
		if (fileTmxs == null) {
			throw new IllegalArgumentException("The pathname does not denote directory: " + dirTmx.toString());
		}

		fListTuvEntry = new ArrayList<TmTuvEntry>();

		for (int index = 0; index < fileTmxs.length; index++) {
			final File fileTmx = fileTmxs[index];
			if (fileTmx.isFile() == false) {
				continue;
			}
			if (fileTmx.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_TMX) == false) {
				// ファイルの拡張子が .tmx のものだけ処理します。
				continue;
			}

			final BlancoTmxParser parser = new BlancoTmxParser();
			final BlancoTmx tmx = parser.parse(fileTmx);

			for (BlancoTmxTu tu : tmx.getBody().getTuList()) {
				final TmTuvEntry entry = new TmTuvEntry();

				String en = null;
				String ja = null;
				for (BlancoTmxTuv tuv : tu.getTuvList()) {
					if (tuv.getLang().toLowerCase().startsWith("en")) { //$NON-NLS-1$
						en = tuv.getSeg();
					} else if (tuv.getLang().toLowerCase().startsWith("ja")) { //$NON-NLS-1$
						ja = tuv.getSeg();
					}
				}
				if (en == null) {
					continue;
				}
				if (ja == null) {
					continue;
				}

				fListTuvEntry.add(entry);
				entry.source = en;
				entry.sourceLower = en.toLowerCase();
				entry.target = ja;
				entry.origin = stripExtension(fileTmx.getName());
			}
		}
	}

	public List<BentenTmSearchResult> fuzzySearch(String target) {
		String regPatternString;
		boolean isRegexMode = false;
		if (target.startsWith("!!")) { //$NON-NLS-1$
			isRegexMode = true;
			// !! の場合に、特殊モードに入ります。
			regPatternString = target.substring(2);
		} else {
			regPatternString = ".*" + Pattern.quote(target.toLowerCase()) + ".*"; //$NON-NLS-1$ //$NON-NLS-2$
		}

		final Pattern regPattern = Pattern.compile(regPatternString);

		final List<BentenTmSearchResult> result = new ArrayList<BentenTmSearchResult>();
		for (int indexTm = 0; indexTm < fListTuvEntry.size(); indexTm++) {
			final TmTuvEntry entry = fListTuvEntry.get(indexTm);

			String searchString = null;
			if (isRegexMode) {
				searchString = entry.source;
			} else {
				searchString = entry.sourceLower;
			}
			final Matcher regMatcher = regPattern.matcher(searchString);
			if (regMatcher.matches()) {
				// ヒット
				final BentenTmSearchResult resultItem = new BentenTmSearchResult();
				result.add(resultItem);
				resultItem.setSource(entry.source);
				resultItem.setTarget(entry.target);
				resultItem.setOrigin(entry.origin);

				int quality = (int) (((double) target.length()) / resultItem.getSource().length() * 100);
				if (resultItem.getSource().equals(target)) {
					quality = 100;
				} else {
					if (quality >= 100) {
						quality = 99;
					}
				}

				resultItem.setMatchQuality(Integer.toString(quality) + "%"); //$NON-NLS-1$
			}
			if (result.size() > MAX_RESULT_SIZE) {
				break;
			}
		}

		Collections.sort(result, new Comparator<BentenTmSearchResult>() {
			public int compare(final BentenTmSearchResult tuLeft, final BentenTmSearchResult tuRight) {
				String left = tuLeft.getMatchQuality().trim();
				String right = tuRight.getMatchQuality().trim();
				if (left.endsWith("%")) { //$NON-NLS-1$
					left = left.substring(0, left.length() - 1);
				}
				if (right.endsWith("%")) { //$NON-NLS-1$
					right = right.substring(0, right.length() - 1);
				}
				return Integer.parseInt(right) - Integer.parseInt(left);
			}
		});

		return result;
	}

	public void unload() {
		fListTuvEntry = new ArrayList<TmTuvEntry>();
	}

	private String stripExtension(String target) {
		if (target == null) {
			return target;
		}

		if (!target.contains(".")) { //$NON-NLS-1$
			return target;
		}

		final Matcher m = extensionPattern.matcher(target);
		m.find();
		return target.substring(0, m.start());
	}
}

class TmTuvEntry {
	public String sourceLower;
	public String source;
	public String target;
	public String origin;
}