/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.engine.regex.ui;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.osgi.framework.BundleContext;

import benten.cat.tm.core.BentenTmDriver;
import benten.cat.tm.core.BentenTmDriverManager;
import benten.cat.tm.core.BentenTmEngine;
import benten.cat.tm.data.AbstractBentenTmDataTmxPlugin;
import benten.cat.tm.engine.AbstractBentenTmEnginePlugin;
import benten.cat.tm.engine.regex.RegexTmDriver;

/**
 * このクラスは、TM エンジンの切り替えテストのためのテストクラスです。
 * 実際の Benten 基本セットには含まれません。
 */
public class RegexTmPlugin extends AbstractBentenTmEnginePlugin {
	private static final boolean IS_DEBUG = false;

	private static final String DEFAULT_TMX = "default tmx"; //$NON-NLS-1$

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	@Override
	public BentenTmDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.tm.engine.regex.RegexTmDriver"); //$NON-NLS-1$
		} catch (final ClassNotFoundException e) {
			log("Regex 翻訳メモリー･エンジンの読み込みに失敗しました。");
			log(e);
		}

		return BentenTmDriverManager.getDriver(RegexTmDriver.DRIVER_NAME);
	}

	/**
	 * TM エンジンのインスタンスを取得します。
	 * 
	 * @return TM エンジンのインスタンス。
	 */
	@Override
	public BentenTmEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}

		return fEngine;
	}

	/**
	 * TM データをロードします。
	 * 
	 * <UL>
	 * <LI>最初にエンジンが保持している TM データをアンロードします。
	 * <LI>拡張ポイント登録一覧から、この TM エンジンが処理可能な拡張ポイントを取得します。
	 * <LI>得られた拡張ポイントから処理可能な拡張のリストを取得します。
	 * <LI>拡張のリストの中で、この TM エンジンが期待する名称の拡張を取得します。
	 * <LI>得られた拡張から、TMX データを読み込みます。
	 * </UL>
	 */
	@Override
	public void load() {
		getDefault().log("Regex TM エンジン・プラグイン: TM をリフレッシュ・ロードします。");

		getEngine().unload();

		final String tmxDir = getTmxDirPath();
		if (tmxDir != null) {
			// TODO null が戻るのは困ります。
			try {
				getEngine().loadTmx(new File(tmxDir));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@SuppressWarnings("static-access")
	private String getTmxDirPath() {
		String result = null;
		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.tm.data.tmx"); //$NON-NLS-1$

		if (IS_DEBUG) {
			System.out.println(point);
		}

		if (point == null)
			return result;

		final Map<String, AbstractBentenTmDataTmxPlugin> tmxMap = new HashMap<String, AbstractBentenTmDataTmxPlugin>();

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					Object tmxPluginClass;
					tmxPluginClass = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (tmxPluginClass instanceof Plugin) {
						tmxMap.put(name, (AbstractBentenTmDataTmxPlugin) tmxPluginClass);
					}
				} catch (final CoreException e) {
					getDefault().log(e);
				}
			}

		}

		final AbstractBentenTmDataTmxPlugin tmxPlugin = tmxMap.get(DEFAULT_TMX);
		if (tmxPlugin == null)
			return result;

		// FIXME ファイルの読み込み方
		final URL entry = tmxPlugin.getDefault().getBundle().getEntry("/tmx"); //$NON-NLS-1$
		try {
			result = FileLocator.resolve(entry).getPath();
		} catch (final IOException e) {
			getDefault().log(e);
		}
		getDefault().log("Directory contains TMX: " + result);

		return result;
	}
}
