/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.ui.preference;

import java.io.File;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import benten.cat.tm.ui.CatTmUiPlugin;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;
import benten.ui.fields.AbstractBrowseTextField;
import benten.ui.fields.FolderTextField;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;
import benten.ui.views.UiToolkit;

/**
 * TM 設定ページです。
 */
public class CatTmUiPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private final UiToolkit toolkit = new UiToolkit();
	private Combo tmNameCombo;
	private Button useTmxDefaultCheck;
	private Button useTmxLocationCheck;
	private AbstractBrowseTextField tmxLocationText;
	private Button ignoreWhitespaceTmReferenceCheck;
	private Button ignoreWhitespaceTmxExtractCheck;

	public CatTmUiPreferencePage() {
	}

	public CatTmUiPreferencePage(String title) {
		super(title);
	}

	public CatTmUiPreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent) {
		final Composite composite = toolkit.createComposite(parent, 1);
		final IPreferenceStore store = BentenPreference.getStore();

		final Composite c = toolkit.createComposite(composite, 2);
		c.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		final Set<String> tmNameSet = CatTmUiPlugin.getDefault().getTmNameSet();
		toolkit.createLabel(c, "翻訳メモリー:");
		tmNameCombo = new Combo(c, SWT.READ_ONLY);
		tmNameCombo.setItems(tmNameSet.toArray(new String[tmNameSet.size()]));
		tmNameCombo.setText(store.getString(Preference.TM_NAME.name()));

		createTmxGroup(composite);
		createWhitespaceGroup(composite);

		return composite;
	}

	protected void createTmxGroup(Composite parent) {
		final IPreferenceStore store = BentenPreference.getStore();

		final Group g = new Group(parent, SWT.BOLD);
		g.setLayout(toolkit.createGridLayout(4, 5));
		g.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		g.setText("使用する TMX のロケーション");

		final GridData gd = new GridData();
		gd.horizontalSpan = 4;
		useTmxDefaultCheck = new Button(g, SWT.RADIO);
		useTmxDefaultCheck.setText("プラグイン組み込み TMX");
		useTmxDefaultCheck.setSelection(store.getBoolean(Preference.USE_TMX_DEFAULT.name()));
		useTmxDefaultCheck.setLayoutData(gd);
		useTmxDefaultCheck.addSelectionListener(new TmxSelectionAdapter());

		useTmxLocationCheck = new Button(g, SWT.RADIO);
		useTmxLocationCheck.setText("次のロケーション:");
		useTmxLocationCheck.setSelection(store.getBoolean(Preference.USE_TMX_LOCATION.name()));
		useTmxLocationCheck.addSelectionListener(new TmxSelectionAdapter());

		tmxLocationText = new FolderTextField(g);
		tmxLocationText.setWidthHint(SWT.DEFAULT);
		tmxLocationText.setText(store.getString(Preference.TMX_LOCATION.name()));

		if (!useTmxDefaultCheck.getSelection() && !useTmxLocationCheck.getSelection()) {
			useTmxDefaultCheck.setSelection(true);
		}
		updateTmxCheck();
	}

	private class TmxSelectionAdapter extends SelectionAdapter {
		@Override
		public void widgetSelected(SelectionEvent e) {
			updateTmxCheck();
		}
	}

	protected void updateTmxCheck() {
		tmxLocationText.setEnabled(useTmxLocationCheck.getSelection());
	}

	protected void createWhitespaceGroup(Composite parent) {
		final IPreferenceStore store = BentenPreference.getStore();

		final Group g = new Group(parent, SWT.BOLD);
		g.setLayout(toolkit.createGridLayout(1, 5));
		g.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		g.setText("空白や改行の扱い");

		ignoreWhitespaceTmReferenceCheck = new Button(g, SWT.CHECK);
		ignoreWhitespaceTmReferenceCheck.setText("TM 参照時に空白や改行を無視(&R)");
		ignoreWhitespaceTmReferenceCheck.setSelection(store
				.getBoolean(Preference.IGNORE_WHITESPACE_TM_REFERENCE.name()));

		ignoreWhitespaceTmxExtractCheck = new Button(g, SWT.CHECK);
		ignoreWhitespaceTmxExtractCheck.setText("TMX 抽出時に空白や改行を無視(&E)");
		ignoreWhitespaceTmxExtractCheck.setSelection(store.getBoolean(Preference.IGNORE_WHITESPACE_TMX_EXTRACT.name()));
	}

	public void init(IWorkbench workbench) {
	}

	@Override
	public boolean performOk() {
		final IPreferenceStore store = BentenPreference.getStore();

		final String newTmName = tmNameCombo.getText();
		final boolean newUseTmxDefault = useTmxDefaultCheck.getSelection();
		final boolean newUseTmxLocation = useTmxLocationCheck.getSelection();

		// TMX ロケーションのチェック
		if (newUseTmxLocation) {
			if (tmxLocationText.getText().equals("")) {
				final UiStatus s = new UiStatus(IStatus.ERROR, "TMX ロケーションを指定してください。");
				UiPlugin.openDialog("入力エラー", s);
				return false;
			}
			final File tmxDir = tmxLocationText.getFile();
			if (!tmxDir.exists() || tmxDir.isFile()) {
				final UiStatus s = new UiStatus(IStatus.ERROR, "指定された TMX ディレクトリーは存在しません。");
				UiPlugin.openDialog("入力エラー", s);
				return false;
			}
		}

		final String oldTmName = store.getString(Preference.TM_NAME.name());
		final boolean oldUseTmxDefault = store.getBoolean(Preference.USE_TMX_DEFAULT.name());
		final boolean oldUseTmxLocation = store.getBoolean(Preference.USE_TMX_LOCATION.name());
		final String oldTmxLocation = store.getString(Preference.TMX_LOCATION.name());

		// 設定が load メソッド内で参照されるため、先に保管
		store.setValue(Preference.USE_TMX_DEFAULT.name(), useTmxDefaultCheck.getSelection());
		store.setValue(Preference.USE_TMX_LOCATION.name(), useTmxLocationCheck.getSelection());
		store.setValue(Preference.TMX_LOCATION.name(), tmxLocationText.getText());

		if (!newTmName.equals(oldTmName)) {

			// TM 名が変更された場合は拡張プラグインをロードしなおす
			store.setValue(Preference.TM_NAME.name(), newTmName);
			CatTmUiPlugin.getDefault().loadExtensions();

		} else {

			// 使用する TMX が変更された場合は TMX をロードしなおす
			if (newUseTmxDefault != oldUseTmxDefault || newUseTmxLocation) {
				try {
					CatTmUiPlugin.getDefault().getTmEnginePlugin().load();

				} catch (final Exception e) {

					// エラーの場合は設定を戻す
					useTmxDefaultCheck.setSelection(oldUseTmxDefault);
					useTmxLocationCheck.setSelection(oldUseTmxLocation);
					tmxLocationText.setText(oldTmxLocation);
					final UiStatus s = new UiStatus(IStatus.ERROR, e.getMessage() + "\n有効な TM 設定を行ってください。");
					UiPlugin.openDialog("TM のロードに失敗しました", s);
					return false;
				}
			}
		}

		store.setValue(Preference.IGNORE_WHITESPACE_TM_REFERENCE.name(), ignoreWhitespaceTmReferenceCheck
				.getSelection());
		store.setValue(Preference.IGNORE_WHITESPACE_TMX_EXTRACT.name(), ignoreWhitespaceTmxExtractCheck.getSelection());
		return true;
	}

	@Override
	protected void performDefaults() {
		final IPreferenceStore store = BentenPreference.getStore();
		tmNameCombo.setText(store.getDefaultString(Preference.TM_NAME.name()));

		useTmxDefaultCheck.setSelection(store.getDefaultBoolean(Preference.USE_TMX_DEFAULT.name()));
		useTmxLocationCheck.setSelection(store.getDefaultBoolean(Preference.USE_TMX_LOCATION.name()));
		tmxLocationText.setText(store.getDefaultString(Preference.TMX_LOCATION.name()));

		ignoreWhitespaceTmReferenceCheck.setSelection(store.getDefaultBoolean(Preference.IGNORE_WHITESPACE_TM_REFERENCE
				.name()));
		ignoreWhitespaceTmxExtractCheck.setSelection(store.getDefaultBoolean(Preference.IGNORE_WHITESPACE_TMX_EXTRACT
				.name()));
	}
}
