/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.validation.validator;

import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.operations.TaskListHelper;
import org.eclipse.wst.validation.internal.provisional.core.IMessage;
import org.eclipse.wst.validation.internal.provisional.core.IReporter;
import org.eclipse.wst.validation.internal.provisional.core.IValidator;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMModel;

import benten.cat.validation.CatValidationPlugin;

/**
 * 検証メッセージを追加するための翻訳リポーターです。
 * @author KASHIHARA Shinji
 */
@SuppressWarnings( { "restriction", "deprecation" })
public class TranslationValidationReporter {

	private final IValidator validator;
	private final IReporter reporter;
	private final IFile file;
	private final IDOMModel model;

	/**
	 * コンストラクター。
	 * @param validator バリデーター
	 * @param reporter レポーター
	 * @param file ファイル
	 * @param model モデル
	 */
	public TranslationValidationReporter(IValidator validator, IReporter reporter, IFile file, IDOMModel model) {
		this.validator = validator;
		this.reporter = reporter;
		this.file = file;
		this.model = model;
	}

	/**
	 * 検証結果のクリア。
	 */
	public void clear() {
		if (file == null) {
			return;
		}
		if (reporter != null) {
			reporter.removeAllMessages(validator, file);
		} else {
			// remove by myself?
			final String id = TranslationValidator.class.getName();
			try {
				// TaskListHelper API changed
				// TaskListHelper.getTaskList().removeAllTasks(id, this.file,
				// null);
				TaskListHelper.getTaskList().removeAllTasks(file, id, null);
			} catch (final CoreException ex) {
				CatValidationPlugin.getDefault().log(ex);
			}
		}
	}

	/**
	 * モデルの取得。
	 * @return モデル
	 */
	public IDOMModel getModel() {
		return model;
	}

	/**
	 * メッセージ･リストの取得。
	 * @return メッセージ･リスト
	 */
	@SuppressWarnings("unchecked")
	public List<IMessage> getMessages() {
		return reporter.getMessages();
	}

	/**
	 * 情報の追加。
	 * @param targetOffset ターゲットのオフセット
	 * @param length 文字数
	 * @param id id
	 * @param params パラメーター
	 */
	public void addInfo(int targetOffset, int length, String id, String... params) {
		add(IMessage.LOW_SEVERITY, targetOffset, length, id, params);
	}

	/**
	 * 警告の追加。
	 * @param targetOffset ターゲットのオフセット
	 * @param length 文字数
	 * @param id id
	 * @param params パラメーター
	 */
	public void addWarning(int targetOffset, int length, String id, String... params) {
		add(IMessage.NORMAL_SEVERITY, targetOffset, length, id, params);
	}

	/**
	 * エラーの追加。
	 * @param targetOffset ターゲットのオフセット
	 * @param length 文字数
	 * @param id id
	 * @param params パラメーター
	 */
	public void addError(int targetOffset, int length, String id, String... params) {
		add(IMessage.HIGH_SEVERITY, targetOffset, length, id, params);
	}

	private void add(int serverity, int targetOffset, int length, String id, String... params) {
		final Message message = new MessageForThisClassLoader();
		message.setSeverity(serverity);
		message.setBundleName("message"); //$NON-NLS-1$
		message.setTargetObject(file);
		message.setGroupName("翻訳検証グループ");
		message.setOffset(targetOffset);
		message.setLength(length);
		message.setLineNo(getLineNumbe(model.getStructuredDocument(), targetOffset));
		message.setId(id);
		message.setParams(params);
		reporter.addMessage(validator, message);
	}

	private int getLineNumbe(IDocument iDoc, int targetOffset) {
		try {
			return iDoc.getLineOfOffset(targetOffset) + 1;
		} catch (final BadLocationException e) {
			return IMessage.LINENO_UNSET;
		}
	}

	/**
	 * メッセージ・クラスです。<br>
	 * 空継承しているのは、setBundleName で指定したプロパティー・ファイルをロードするときに
	 * メッセージ・クラスから getClass().getClassLoader() し、そのクラス・ローダーから
	 * プラグインのリソースが読み込まれるためです。
	 */
	private static class MessageForThisClassLoader extends Message {
	}
}
