/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

import benten.core.BentenConstants;
import benten.twa.cat.core.valueobject.BentenApplyExistentTranslationProcessInput;
import benten.twa.cat.messages.BentenApplyExistentTranslationMessages;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import blanco.commons.util.BlancoFileUtil;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * 過去訳の適用
 *
 * <pre>
 * 過去訳を XLIFF に適用します。
 *  1.  翻訳単位の id をもとに過去訳を適用します。
 *  2.  翻訳後の XLIFF にある過去訳を、XLIFF に適用します。
 *  3.  この処理では TMX を利用しません。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳中間形式機械支援翻訳機能: 過去訳適用機能」に対応します。
 */
public class BentenApplyExistentTranslationProcessImpl extends AbstractTraverseDir implements
		BentenApplyExistentTranslationProcess {
	/**
	 * 過去訳適用機能のためのメッセージ。
	 */
	protected static final BentenApplyExistentTranslationMessages fMsg = new BentenApplyExistentTranslationMessages();

	protected BentenApplyExistentTranslationProcessInput fInput;

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenApplyExistentTranslationProcessInput input) throws IOException,
			IllegalArgumentException {
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		processDir(new File(input.getSourcedir()));

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(final String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	protected void processFile(final File fileCurrent, final String baseDir) throws IOException {
		if (progress(fMsg.getCoreP011(fileCurrent.getName()))) {
			return;
		}

		final File fileTarget = new File(fInput.getTargetdir() + "/" + baseDir + fileCurrent.getName()); //$NON-NLS-1$

		// TODO 同名ファイルのみを処理対象としているが、この考えが妥当かどうかを検証する必要あり。
		// TODO というか、同名ファイルのみ適用は、あまり妥当ではない。
		// TODO XLIFF マージと同様、クロスでネストさせるのが良いと考える。

		if (fileTarget.exists() == false) {
			System.out.println(fMsg.getCoreW001(fileCurrent.getName()));
			fResultInfo.setWarningCount(fResultInfo.getWarningCount() + 1);
			fResultInfo.getMessageList().add(fMsg.getCoreW001(fileCurrent.getName()));
			return;
		}
		if (fileTarget.isFile() == false) {
			System.out.println(fMsg.getCoreW002(fileCurrent.getName()));
			fResultInfo.setWarningCount(fResultInfo.getWarningCount() + 1);
			fResultInfo.getMessageList().add(fMsg.getCoreW002(fileCurrent.getName()));
			return;
		}

		BlancoXliff xliffFrom = null;
		try {
			xliffFrom = new BlancoXliffParser().parse(fileCurrent);
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE011(fileCurrent.getName(), e.toString()));
		}

		BlancoXliff xliffTo = null;
		try {
			xliffTo = new BlancoXliffParser().parse(fileTarget);
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE012(fileCurrent.getName(), e.toString()));
		}

		final Map<String, BlancoXliffTransUnit> map = new HashMap<String, BlancoXliffTransUnit>(19384);
		for (BlancoXliffFile toFile : xliffTo.getFileList()) {
			for (BlancoXliffTransUnit transUnitTo : toFile.getBody().getTransUnitList()) {
				map.put(transUnitTo.getId(), transUnitTo);
			}
		}

		int transUnitTotalCount = 0;
		for (BlancoXliffFile fromFile : xliffFrom.getFileList()) {
			transUnitTotalCount += fromFile.getBody().getTransUnitList().size();
		}
		beginTask(transUnitTotalCount);

		int transUnitCount = 0;
		for (BlancoXliffFile fromFile : xliffFrom.getFileList()) {
			for (BlancoXliffTransUnit transUnitFrom : fromFile.getBody().getTransUnitList()) {
				if (progress(fMsg.getCoreP101(fileCurrent.getName(), BigDecimal.valueOf(++transUnitCount), BigDecimal
						.valueOf(transUnitTotalCount)))) {
					break;
				}

				if (map.get(transUnitFrom.getId()) == null) {
					fResultInfo.getMessageList().add(fMsg.getCoreI001(transUnitFrom.getId()));
					System.out.println(fMsg.getCoreI001(transUnitFrom.getId()));
					continue;
				}

				final BlancoXliffTransUnit transUnitTo = map.get(transUnitFrom.getId());

				if (transUnitTo.getTarget() != null && transUnitTo.getTarget().getTarget() != null
						&& transUnitTo.getTarget().getTarget().length() > 0) {
					continue;
				}

				fResultInfo.setUnitCount(fResultInfo.getUnitCount() + 1);

				// 過去訳適用では、基本的に target 要素の内容のみを適用します。
				// 適用先に target が既に存在する場合には、これの適用をあきらめます。
				// このため、XLIFF 作成直後の適用が望ましい。
				// マージとの差違は、target 以外をコピーしない点である。
				// target 以外には translate="no" と翻訳メモリ反映対象外が転送される。
				// 翻訳を適用した場合には、alt-trans に過去訳を格納する。
				// TODO state は何にする? translated ???
				// alt-trans を複写したい場合には、XLIFF マージを別途実施する。

				// TODO alt-trans や note をコピーするかどうかの検討が必要。

				// TODO 単に上書きしてしまっているが、これが妥当かどうか検討したい。
				// TODO たとえば、state で優先順位があり、強い方が勝つ、など。
				// TODO また、この動作と XLIFF マージの動作とが非常に似ている。

				// TODO XLIFF マージとの差違を検討しなおすこと。
				// TODO 少なくとも、過去訳適用の差違には alt-trans に "prev" で過去訳を保持する。などの仕様があろう。

				// TODO alt-trans のコピー。
				// alt-trans は "benten" のものはコピーしません。

				transUnitTo.setTarget(transUnitFrom.getTarget());
			}
		}

		if (progress(fMsg.getCoreP012(fileTarget.getName()))) {
			return;
		}

		// 書き出し。
		try {
			final ByteArrayOutputStream outStream = new ByteArrayOutputStream();
			new BlancoXliffSerializer().serialize(xliffTo, outStream);
			outStream.flush();

			if (BlancoFileUtil.bytes2FileIfNecessary(outStream.toByteArray(), fileTarget) == 0) {
				// 変更無し。
				getResultInfo().setSkipCount(getResultInfo().getSkipCount() + 1);
			} else {
				// 更新のあったファイル数をカウントします。
				getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
			}
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE013(fileTarget.getName(), e.toString()));
		}

		if (progress(fMsg.getCoreP013(fileTarget.getName()))) {
			return;
		}
	}
}
