/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.cat.core;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;

import benten.core.BentenConstants;
import benten.core.io.IniFile;
import benten.twa.cat.core.valueobject.BentenMarkNoTransableProcessInput;
import benten.twa.cat.messages.BentenMarkNoTransableMessages;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import blanco.commons.util.BlancoFileUtil;
import blanco.xliff.BlancoXliffParser;
import blanco.xliff.BlancoXliffSerializer;
import blanco.xliff.valueobject.BlancoXliff;
import blanco.xliff.valueobject.BlancoXliffFile;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * 非翻訳対象のマーク
 *
 * <pre>
 * XLIFF 内の翻訳の必要のない翻訳単位を、非翻訳対象としてマークします。
 *   1.  XLIFF の翻訳単位の内容から、翻訳の必要の有無を判断します。
 *   2.  翻訳の必要がないと判断されたものについて、非翻訳対象としてマークします。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳中間形式機械支援翻訳機能: 非翻訳対象マーク機能」に対応します。
 */
public class BentenMarkNoTransableProcessImpl extends AbstractTraverseDir implements BentenMarkNoTransableProcess {
	/**
	 * 非翻訳対象マーク機能のためのメッセージ。
	 */
	protected static final BentenMarkNoTransableMessages fMsg = new BentenMarkNoTransableMessages();

	protected BentenMarkNoTransableProcessInput fInput;

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	protected final List<String> markNoTransableList;

	public BentenMarkNoTransableProcessImpl() {
		markNoTransableList = IniFile.asList(this, "conf/mark-no-transable.ini"); //$NON-NLS-1$
	}

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenMarkNoTransableProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		processDir(new File(input.getXliffdir()));

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(final File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	protected void processFile(final File fileCurrent, final String baseDir) throws IOException {
		if (progress(fMsg.getCoreP011(fileCurrent.getName()))) {
			return;
		}

		BlancoXliff xliff = null;
		try {
			xliff = new BlancoXliffParser().parse(fileCurrent);
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE011(fileCurrent.getName(), e.toString()));
		}

		int transUnitTotalCount = 0;
		for (final BlancoXliffFile file : xliff.getFileList()) {
			transUnitTotalCount += file.getBody().getTransUnitList().size();
		}
		beginTask(transUnitTotalCount);

		int transUnitCount = 0;
		for (final BlancoXliffFile file : xliff.getFileList()) {
			for (final BlancoXliffTransUnit transUnit : file.getBody().getTransUnitList()) {
				if (progress(fMsg.getCoreP101(fileCurrent.getName(), BigDecimal.valueOf(++transUnitCount), BigDecimal
						.valueOf(transUnitTotalCount)))) {
					break;
				}

				processTransUnit(transUnit);
			}
		}

		if (progress(fMsg.getCoreP012(fileCurrent.getName()))) {
			return;
		}

		// 書き出し。
		try {
			final ByteArrayOutputStream outStream = new ByteArrayOutputStream();
			new BlancoXliffSerializer().serialize(xliff, outStream);
			outStream.flush();

			if (BlancoFileUtil.bytes2FileIfNecessary(outStream.toByteArray(), fileCurrent) == 0) {
				// 変更無し。
				getResultInfo().setSkipCount(getResultInfo().getSkipCount() + 1);
			} else {
				// 更新のあったファイル数をカウントします。
				getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
			}
		} catch (IllegalArgumentException e) {
			throw new IOException(fMsg.getCoreE012(fileCurrent.getName(), e.toString()));
		}

		if (progress(fMsg.getCoreP013(fileCurrent.getName()))) {
			return;
		}
	}

	protected void processTransUnit(final BlancoXliffTransUnit transUnit) {
		if (transUnit.getTranslate() == false) {
			// すでに除外されているので、これ以上判断する必要はありません。
			return;
		}
		if (transUnit.getTarget() != null && transUnit.getTarget().getTarget() != null) {
			if (transUnit.getTarget().getTarget().length() > 0) {
				// trans-unit の target に何かしらの値が設定済みのもにについては、
				// 何か意図があって設定されているものとみなし、
				// 除外マーク判定の対象外とします。
				return;
			}
		}

		// 翻訳が不要な文字列かどうか判定します。
		if (checkNoTransableString(transUnit.getSource())) {
			fResultInfo.setUnitCount(fResultInfo.getUnitCount() + 1);
			transUnit.setTranslate(false);
			return;
		}
	}

	/**
	 * 翻訳が不要な文字列かどうか判定します。
	 *
	 * @param argCheckTarget
	 * @return
	 */
	protected boolean checkNoTransableString(final String argCheckTarget) {
		if (argCheckTarget == null) {
			return true;
		}

		final String trimmedCheckTarget = argCheckTarget.trim();

		// 翻訳不要マーク定義リストをもとに翻訳不要か判断します。
		for (final String pattern : markNoTransableList) {
			if (trimmedCheckTarget.matches(pattern)) {

				// 翻訳不要マーク定義パターンに一致した場合は翻訳不要です。
				return true;
			}
		}

		// 翻訳が必要です。
		return false;
	}
}
