/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.core;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import benten.core.BentenConstants;
import benten.core.io.Files;
import benten.core.model.BentenXliff;
import benten.core.model.HelpTransUnitId;
import benten.twa.filter.core.valueobject.BentenExportHtmlProcessInput;
import benten.twa.filter.messages.BentenExportHtmlMessages;
import benten.twa.filter.model.HtmlInlineTextBuilder;
import benten.twa.filter.model.HtmlInlineTextBuilder.InlineText;
import benten.twa.io.AbstractTraverseDir;
import benten.twa.process.BentenProcessResultInfo;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * 翻訳成果のエクスポート
 * 
 * <pre>
 * 翻訳成果物をプロジェクトからエクスポートします。
 *   1.  XLIFF とインポートされた翻訳対象物をもちいて、翻訳成果物をエクスポートします。
 * </pre>
 * 
 * ★基本設計「翻訳ワークフロー支援機能: 翻訳対象物－翻訳中間形式変換機能: HTMLエクスポート機能」に対応します。
 */
public class BentenExportHtmlProcessImpl extends AbstractTraverseDir implements BentenExportHtmlProcess {
	/**
	 * HTMLエクスポート機能のためのメッセージ。
	 */
	protected static final BentenExportHtmlMessages fMsg = new BentenExportHtmlMessages();

	protected BentenExportHtmlProcessInput fInput;

	/**
	 * この処理の実行結果情報。
	 */
	protected BentenProcessResultInfo fResultInfo = new BentenProcessResultInfo();

	public void setInput(BentenExportHtmlProcessInput input) {
		fInput = input;
	}

	/**
	 * この処理の実行結果情報を取得します。
	 * 
	 * @return 処理結果情報。
	 */
	public BentenProcessResultInfo getResultInfo() {
		return fResultInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	public int execute(final BentenExportHtmlProcessInput input) throws IOException, IllegalArgumentException {
		fInput = input;

		if (progress(fMsg.getCoreP001())) {
			return 6;
		}

		// トータル件数カウント。
		class FileCounter extends BentenExportHtmlProcessImpl {
			private int fCounter = 0;

			@Override
			public void processFile(File htmlFile, String baseDir) throws IOException {
				// 処理進捗が 2 度呼ばれます。
				fCounter += 2;
			}

			@Override
			protected void processFilterdFile(final File file, final String baseDir) throws IOException {
				fCounter++;
			}

			public int getCounter() {
				return fCounter;
			}
		}
		final FileCounter inner = new FileCounter();
		inner.setInput(input);
		inner.processDir(new File(input.getSourcexliffdir()));

		// トータル件数カウント。
		class FileCounter2 extends BentenExportHtmlProcessImpl {
			private int fCounter = 0;

			@Override
			protected boolean canProcess(File file) {
				return !(file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase() //$NON-NLS-1$
						.endsWith(".htm")); //$NON-NLS-1$
			}

			@Override
			public void processFile(File htmlFile, String baseDir) throws IOException {
				fCounter += 1;
			}

			public int getCounter() {
				return fCounter;
			}
		}
		final FileCounter2 inner2 = new FileCounter2();
		inner2.setInput(input);
		inner2.processDir(new File(input.getSourcehtmldir()));

		beginTask(inner.getCounter() + inner2.getCounter());

		if (progress(fMsg.getCoreP002())) {
			return 6;
		}

		processDir(new File(input.getSourcexliffdir()));

		if (progress(fMsg.getCoreP003())) {
			return 6;
		}

		// HTML 以外をコピー
		new AbstractTraverseDir() {
			{
				processDir(new File(fInput.getSourcehtmldir()));
			}

			@Override
			protected void beginTask(final int totalWork) {
			}

			@Override
			protected boolean canProcess(File file) {
				return !(file.getName().toLowerCase().endsWith(".html") || file.getName().toLowerCase() //$NON-NLS-1$
						.endsWith(".htm")); //$NON-NLS-1$
			}

			@Override
			protected void processFile(File file, String baseDir) throws IOException {
				if (progress(fMsg.getCoreP013(file.getName()))) {
					return;
				}

				final String relativePath = Files.relativePath(new File(fInput.getSourcehtmldir()), file);
				final File outputFile = new File(fInput.getTargetdir(), relativePath);
				Files.copyFile(file, outputFile);

				getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
			}
		};

		if (progress(fMsg.getCoreP004())) {
			return 6;
		}

		return 0;
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean progress(String argProgressMessage) {
		if (fInput != null && fInput.getVerbose()) {
			System.out.println(argProgressMessage);
		}
		return false;
	}

	@Override
	protected boolean canProcess(File file) {
		return file.getName().toLowerCase().endsWith(BentenConstants.FILE_EXT_XLIFF);
	}

	@Override
	public void processFile(File xliffFile, String baseDir) throws IOException {
		if (progress(fMsg.getCoreP011(xliffFile.getName()))) {
			return;
		}

		final String relativePath = Files.relativePath(new File(fInput.getSourcexliffdir()), xliffFile);
		final File toDir = new File(fInput.getTargetdir(), relativePath).getParentFile();
		final File htmlDir = new File(fInput.getSourcehtmldir(), relativePath).getParentFile();

		writeHtmlFromXliff(xliffFile, htmlDir, toDir);
	}

	protected void writeHtmlFromXliff(final File xliffFile, final File htmlDir, final File toDir) throws IOException {
		BentenXliff xliff = BentenXliff.loadInstance(xliffFile);
		Iterator<BlancoXliffTransUnit> units = xliff.getAllTransUnitList().iterator();
		String htmlFileName = Files.baseName(xliffFile);
		File inHtmlFile = new File(htmlDir, htmlFileName);
		HtmlInlineTextBuilder htmlBuilder = new HtmlInlineTextBuilder(inHtmlFile);

		if (progress(fMsg.getCoreP012(inHtmlFile.getName()))) {
			return;
		}

		for (InlineText inline : htmlBuilder) {
			StringBuilder target = new StringBuilder();

			while (units.hasNext()) {
				fResultInfo.setUnitCount(fResultInfo.getUnitCount() + 1);

				BlancoXliffTransUnit unit = units.next();
				BlancoXliffTarget t = unit.getTarget();

				if (t != null && t.getTarget() != null && !t.getTarget().equals("")) { //$NON-NLS-1$
					target.append(t.getTarget());
				} else {
					target.append(unit.getSource());
				}

				if (!HelpTransUnitId.hasContinue(unit.getId())) {
					break;
				}
			}

			if (target.length() > 0) {
				inline.setText(target.toString());
			} else {
				inline.setText(inline.getText());
			}
		}

		toDir.mkdirs();
		File outHtmlFile = (new File(toDir, htmlFileName));

		htmlBuilder.writeTo(outHtmlFile, "UTF-8"); //$NON-NLS-1$

		getResultInfo().setSuccessCount(getResultInfo().getSuccessCount() + 1);
	}
}
