/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import benten.core.text.Strings;

/**
 * HTML ブロックです。
 */
public class HtmlBlock {

	private static final boolean IS_DEBUG = Boolean.valueOf(System.getProperty("benten.debug")); //$NON-NLS-1$
	private static final Pattern enclosedTagPattern = Pattern.compile("(?i)(?s)^(<[^>]+>\\s*)(.+)(\\s*<[^>]+>)$"); //$NON-NLS-1$
	private final Matcher matcher;
	private final boolean isBlockTag;
	private boolean containesHtmlCommentStartTag;
	private boolean containesHtmlCommentEndTag;
	private final String inlineText;

	/**
	 * コンストラクター。
	 * @param input 入力文字列
	 */
	public HtmlBlock(String input) {
		matcher = enclosedTagPattern.matcher(input);
		isBlockTag = matcher.find();
		if (!isBlockTag) {
			inlineText = ""; //$NON-NLS-1$
			return;
		}

		if (IS_DEBUG) {
			System.out.println(""); //$NON-NLS-1$
			System.out.println("[BLOCK ALL  ] " + Strings.removeRedundantWhitespace(toString())); //$NON-NLS-1$
			System.out.println("[BLOCK START] " + Strings.removeRedundantWhitespace(startTag())); //$NON-NLS-1$
			System.out.println("[BLOCK BODY ] " + Strings.removeRedundantWhitespace(body())); //$NON-NLS-1$
			System.out.println("[BLOCK END  ] " + Strings.removeRedundantWhitespace(endTag())); //$NON-NLS-1$
		}

		inlineText = trim(body());

		if (IS_DEBUG) {
			System.out.println("---------------------------------"); //$NON-NLS-1$
			System.out.println("[After  ] " + Strings.removeRedundantWhitespace(inlineText)); //$NON-NLS-1$
		}
	}

	/**
	 * ブロック・タグを除くテキストを取得します。
	 * @return テキスト
	 */
	public String getInlineText() {
		return inlineText;
	}

	/**
	 * ブロック・タグか判定します。
	 * @return ブロック・タグの場合は true
	 */
	public boolean isBlockTag() {
		return isBlockTag;
	}

	/**
	 * 先頭開始タグと末尾終了タグの要素名が同じか判定します。
	 * @return 同じ場合は true
	 */
	public boolean equalsStartEndTag() {
		if (startTagName().equals(endTagName())) {
			final String pattern = "(?i)(?s).*<" + startTagName() + "(\\s|>).*"; //$NON-NLS-1$ //$NON-NLS-2$
			return !body().matches(pattern);
		}
		return false;
	}

	/**
	 * 先頭の開始タグを取得します。
	 * @return 先頭の開始タグ
	 */
	public String startTag() {
		return matcher.group(1);
	}

	/**
	 * 先頭の開始タグ名を取得します。
	 * @return 先頭の開始タグ名
	 */
	public String startTagName() {
		return getTagName(startTag());
	}

	/**
	 * 先頭末尾のブロック・タグを除いたテキストを取得します。
	 * ブロック・タグが含まれている場合があります。
	 * @return テキスト
	 */
	public String body() {
		return matcher.group(2);
	}

	/**
	 * 末尾の終了タグを取得します。
	 * @return 末尾の終了タグ
	 */
	public String endTag() {
		return matcher.group(3);
	}

	/**
	 * 末尾の終了タグ名を取得します。
	 * @return 末尾の終了タグ名
	 */
	public String endTagName() {
		return getTagName(endTag());
	}

	/**
	 * 末尾の終了タグ位置を取得します。
	 * @return 末尾の終了位置
	 */
	public int endTagIndex() {
		return matcher.start(3);
	}

	private String getTagName(String tag) {
		return tag.trim().replaceAll("([/<>]|\\s\\w.+)", ""); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * HTML コメント開始タグ (<!--) が含まれるか判定します。
	 * @return 含まれる場合は true
	 */
	public boolean containesHtmlCommentStartTag() {
		return containesHtmlCommentStartTag;
	}

	/**
	 * HTML コメント終了タグ (-->) が含まれるか判定します。
	 * @return 含まれる場合は true
	 */
	public boolean containesHtmlCommentEndTag() {
		return containesHtmlCommentEndTag;
	}

	@Override
	public String toString() {
		return startTag() + body() + endTag();
	}

	private String trim(String text) {
		// 前後のブロック・タグ除去
		String s = HtmlBlockMatcher.trimBlockTag(text);

		// HTML コメント処理
		s = removeHtmlComment(s);

		// 読み飛ばし判定
		if (isIgnore(s)) {
			return ""; //$NON-NLS-1$
		}
		// 全体が a タグでない場合 (a タグが片側だけになるのを防止)
		if (!s.matches("(?i)(?s)<a\\s.+?>.*</a>")) { //$NON-NLS-1$

			// 前後のインライン・タグのうち、タグ内のテキストが空の場合は除去
			s = HtmlBlockMatcher.trimTag(s, "<[^</]+>\\s*</[^<]+>"); //$NON-NLS-1$
		}
		// 前後の対応するインライン・タグを除去 (属性に title や alt がある場合は除去しない)
		final HtmlBlock block = new HtmlBlock(s);
		if (block.isBlockTag() && block.equalsStartEndTag()) {
			if (!needsTranslationAttribute(block.startTag())) {
				s = block.body();
			}
		}

		if (s.equals(text)) {
			return s;
		}
		return trim(s); // 再帰
	}

	private String removeHtmlComment(String s) {
		final String HTML_COMMENT_START = "<!--"; //$NON-NLS-1$
		final String HTML_COMMENT_END = "-->"; //$NON-NLS-1$

		if (s.contains(HTML_COMMENT_START) && s.contains(HTML_COMMENT_END)) {
			final String pattern = "(?i)(?s)" + HTML_COMMENT_START + ".*" + HTML_COMMENT_END; //$NON-NLS-1$ //$NON-NLS-2$
			s = s.replaceAll(pattern, ""); //$NON-NLS-1$
		}
		if (s.contains(HTML_COMMENT_START)) {
			final String pattern = "(?i)(?s)" + HTML_COMMENT_START + ".*"; //$NON-NLS-1$ //$NON-NLS-2$
			s = s.replaceFirst(pattern, ""); //$NON-NLS-1$
			containesHtmlCommentStartTag = true;
		} else if (s.contains(HTML_COMMENT_END)) {
			final String pattern = "(?i)(?s)" + ".*?" + HTML_COMMENT_END; //$NON-NLS-1$ //$NON-NLS-2$
			s = s.replaceFirst(pattern, ""); //$NON-NLS-1$
			containesHtmlCommentEndTag = true;
		}
		return s;
	}

	private boolean isIgnore(String s) {
		// 1 文字の場合
		if (s.length() == 1) {
			return true;
		}
		// 全体が 1 つのタグ (テキスト部が無い) 、かつ翻訳が必要な属性が無い場合
		if (s.matches("(?i)(?s)<[^<]+>") && !needsTranslationAttribute(s)) { //$NON-NLS-1$
			return true;
		}
		// 全体が属性の無い 1 つのタグ (つまりテキスト部が無い) の場合
		if (s.matches("(?i)(?s)^<[^<\\s]+>$")) { //$NON-NLS-1$
			return true;
		}
		return false;
	}

	private boolean needsTranslationAttribute(String text) {
		return text.matches("(?i)(?s).+?\\s(title|alt)=\".+"); //$NON-NLS-1$
	}
}
