/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.ui.jobs;

import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;

import benten.twa.TwaPlugin;
import benten.twa.process.BentenProcessResultInfo;
import benten.ui.UiMultiStatus;
import benten.ui.UiPlugin;
import benten.ui.UiStatus;
import benten.ui.jobs.BentenSchedulingRule;

/**
 * 翻訳ワークフロー・ジョブ。
 * @author KASHIHARA Shinji
 */
public class WorkflowJob extends Job {

	private AbstractWorkflowFileHandler handler;
	private IStatus status;
	private String completDialogTitle;
	private String completeMessage;

	/**
	 * コンストラクター。
	 * @param name ジョブ名
	 */
	public WorkflowJob(String name) {
		super(name);
		setUser(true);

		// ワークフロー・ジョブの同時実行はすべてブロック
		setRule(BentenSchedulingRule.INSTANCE);
	}

	/**
	 * ステータスの取得。
	 * @param status ステータス
	 */
	public void setStatus(IStatus status) {
		this.status = status;
	}

	/**
	 * ハンドラーのセット。
	 * @param handler ハンドラー
	 */
	public void setHandler(AbstractWorkflowFileHandler handler) {
		this.handler = handler;
	}

	@Override
	protected IStatus run(IProgressMonitor monitor) {
		handler.monitor = monitor;

		// ハンドラー実行
		try {
			handler.run(status);
		} catch (final Exception e) {
			TwaPlugin.getDefault().log(e);
			return Status.CANCEL_STATUS;
		}

		// ユーザーによるキャンセル
		if (monitor.isCanceled()) {
			return Status.CANCEL_STATUS;
		}

		// 処理結果取得
		final BentenProcessResultInfo resultInfo = handler.getResultInfo();
		final int count = resultInfo.getSuccessCount() + resultInfo.getWarningCount();
		if (count == 0) {

			// エラー表示
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					final UiStatus s = new UiStatus(IStatus.INFO, "処理対象となるファイルがありませんでした。");
					UiPlugin.openDialog("処理対象がありません", s);
				}
			});
			return Status.CANCEL_STATUS;

		} else {

			// プロジェクトのリフレッシュ
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					if (status instanceof UiStatus) {
						final UiStatus uiStatus = (UiStatus) status;
						if (uiStatus.getProject() != null) {
							try {
								uiStatus.getProject().refreshLocal(IResource.DEPTH_INFINITE, null);
							} catch (final CoreException e) {
								TwaPlugin.getDefault().log(e);
							}
						}
					}
				}
			});

			// 完了メッセージ表示
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					showCompleteMessage();
				}
			});

			// ハンドラー完了処理
			handler.complete(status);

			return Status.OK_STATUS;
		}
	}

	/**
	 * この処理の実行結果情報を取得。
	 * @return 処理結果情報
	 */
	public BentenProcessResultInfo getResultInfo() {
		return handler.getResultInfo();
	}

	/**
	 * 完了メッセージの表示。
	 */
	protected void showCompleteMessage() {

		// ダイアログに表示するメッセージの作成
		final StringBuffer sb = new StringBuffer();
		sb.append(getCompleteMessage());
		sb.append("\n");
		sb.append("正常: ");
		sb.append(getResultInfo().getSuccessCount());
		sb.append(" 件");
		if (getResultInfo().getWarningCount() > 0) {
			sb.append("、警告: ");
			sb.append(getResultInfo().getWarningCount());
			sb.append(" 件");
		}
		if (getResultInfo().getSkipCount() > 0) {
			sb.append("、スキップ: ");
			sb.append(getResultInfo().getSkipCount());
			sb.append(" 件");
		}
		if (getResultInfo().getUnitCount() > 0) {
			sb.append("\n単位: ");
			sb.append(getResultInfo().getUnitCount());
			sb.append(" 件");
		}

		// 詳細ボタン押下時のメッセージを作成
		IStatus s = new UiStatus(IStatus.INFO, sb.toString());
		final List<String> messageList = getResultInfo().getMessageList();
		if (messageList.size() > 0) {
			final UiMultiStatus multiStatus = new UiMultiStatus(sb.toString());
			for (final String message : getResultInfo().getMessageList()) {
				multiStatus.add(new UiStatus(IStatus.INFO, message));
			}
			s = multiStatus;
		}

		// ダイアログ・オープン
		UiPlugin.openDialog(getCompleteDialogTitle(), s);
	}

	/**
	 * 完了メッセージのセット。
	 * @param dialogTitle ダイアログ・タイトル
	 * @param message メッセージ
	 */
	public void setCompleteMessage(String dialogTitle, String message) {
		this.completDialogTitle = dialogTitle;
		this.completeMessage = message;
	}

	/**
	 * 完了ダイアログのタイトルを取得。<br>
	 * セットされていない場合はデフォルトのタイトルを返します。
	 * 現状、ウィザードでは個別、ダイアログではデフォルトのタイトルが使用されています。
	 *
	 * @return 完了ダイアログのタイトル
	 */
	protected String getCompleteDialogTitle() {
		return completDialogTitle == null ? "処理が完了しました" : completDialogTitle;
	}

	/**
	 * 完了メッセージを取得。<br>
	 * セットされていない場合はデフォルトのメッセージを返します。
	 * 現状、ウィザードでは個別、ダイアログではデフォルトのメッセージが使用されています。
	 *
	 * @return 完了メッセージ
	 */
	protected String getCompleteMessage() {
		return completeMessage == null ? "処理が完了しました。" : completeMessage;
	}
}
